@@ -238,7 +238,7 @@ bool FileTransferJob::makeDir(const FilePath& srcPath, GFileInfoPtr srcInfo, Fil
                 }
 
                 FilePath newDestPath;
-                FileExistsAction opt = askRename(FileInfo{srcInfo, srcPath.parent()}, FileInfo{destInfo, destPath.parent()}, newDestPath);
+                FileExistsAction opt = askRename(FileInfo{srcInfo, srcPath}, FileInfo{destInfo, destPath}, newDestPath);
                 switch(opt) {
                 case FileOperationJob::RENAME:
                     destPath = std::move(newDestPath);
@@ -311,8 +311,8 @@ bool FileTransferJob::handleError(GErrorPtr &err, const FilePath &srcPath, const
         // ask the user to rename or overwrite the existing file
         if(!isCancelled() && destInfo) {
             FilePath newDestPath;
-            FileExistsAction opt = askRename(FileInfo{srcInfo, srcPath.parent()},
-                                             FileInfo{destInfo, destPath.parent()},
+            FileExistsAction opt = askRename(FileInfo{srcInfo, srcPath},
+                                             FileInfo{destInfo, destPath},
                                              newDestPath);
             switch(opt) {
             case FileOperationJob::RENAME:
@@ -449,7 +449,18 @@ bool FileTransferJob::copyFile(const FilePath& srcPath, const GFileInfoPtr& srcI
     if(!skip) {
         switch(file_type) {
         case G_FILE_TYPE_DIRECTORY:
-            success = makeDir(srcPath, srcInfo, destPath);
+            // prevent a dir from being copied into itself
+            if(srcPath.isPrefixOf(destPath)) {
+                GErrorPtr err = GErrorPtr{
+                                G_IO_ERROR,
+                                G_IO_ERROR_NOT_SUPPORTED,
+                                tr("Cannot copy a directory into itself!")
+                };
+                emitError(err, ErrorSeverity::MODERATE);
+            }
+            else {
+                success = makeDir(srcPath, srcInfo, destPath);
+            }
             break;
         case G_FILE_TYPE_SPECIAL:
             success = copySpecialFile(srcPath, srcInfo, destPath);
