@@ -1,5 +1,6 @@
 #include "iconinfo.h"
 #include "iconinfo_p.h"
+#include <string.h>
 
 namespace Fm {
 
@@ -144,4 +145,32 @@ QIcon IconInfo::internalQicon() const {
     return ret_icon;
 }
 
+// compatibility function for leagcy libfm
+// FIXME: deprecate this later.
+extern "C" GIcon* _fm_icon_from_name(const char* name) {
+    GIcon* gicon = nullptr;
+    if(G_LIKELY(name)) {
+        gchar *dot;
+        if(g_path_is_absolute(name)) {
+            GFile* gicon_file = g_file_new_for_path(name);
+            gicon = g_file_icon_new(gicon_file);
+            g_object_unref(gicon_file);
+        }
+        else if(G_UNLIKELY((dot = strrchr((char*)name, '.')) != NULL && dot > name &&
+                (g_ascii_strcasecmp(&dot[1], "png") == 0
+                 || g_ascii_strcasecmp(&dot[1], "svg") == 0
+                 || g_ascii_strcasecmp(&dot[1], "xpm") == 0))) {
+            /* some desktop entries have invalid icon name which contains
+               suffix so let strip the suffix from such invalid name */
+            dot = g_strndup(name, dot - name);
+            gicon = g_themed_icon_new_with_default_fallbacks(dot);
+            g_free(dot);
+        }
+        else {
+            gicon = g_themed_icon_new_with_default_fallbacks(name);
+        }
+    }
+    return gicon;
+}
+
 } // namespace Fm
