@@ -7,6 +7,8 @@
 #include <QDir>
 #include "thumbnailer.h"
 
+#include "core/legacy/fm-config.h"
+
 namespace Fm {
 
 QThreadPool* ThumbnailJob::threadPool_ = nullptr;
@@ -77,13 +79,24 @@ QImage ThumbnailJob::loadForFile(const std::shared_ptr<const FileInfo> &file) {
 
     // generate base name of the thumbnail  => {md5 of uri}.png
     auto origPath = file->path();
-    auto uri = origPath.uri();
+    CStrPtr uri;
+    if(file->isSymlink()) {
+        // use the symlink target in the name to update the thumbnail
+        // if the file is changed to a symlink with the same time stamp
+        auto target = file->target();
+        if(!target.empty()) {
+            uri = FilePath::fromLocalPath(target.c_str()).uri();
+        }
+    }
+    if(!uri) {
+        uri = origPath.uri();
+    }
 
     char thumbnailName[32 + 5];
     // calculate md5 hash for the uri of the original file
     g_checksum_update(md5Calc_, reinterpret_cast<const unsigned char*>(uri.get()), -1);
     memcpy(thumbnailName, g_checksum_get_string(md5Calc_), 32);
-    mempcpy(thumbnailName + 32, ".png", 5);
+    memcpy(thumbnailName + 32, ".png", 5);
     g_checksum_reset(md5Calc_); // reset the checksum calculator for next use
 
     QString thumbnailFilename = thumbnailDir;
@@ -262,5 +275,19 @@ QThreadPool* ThumbnailJob::threadPool() {
     return threadPool_;
 }
 
+void ThumbnailJob::setLocalFilesOnly(bool value) {
+    localFilesOnly_ = value;
+    if(fm_config) {
+        fm_config->thumbnail_local = localFilesOnly_;
+    }
+}
+
+void ThumbnailJob::setMaxThumbnailFileSize(int size) {
+    maxThumbnailFileSize_ = size;
+    if(fm_config) {
+        fm_config->thumbnail_max = maxThumbnailFileSize_;
+    }
+}
+
 
 } // namespace Fm
