@@ -306,7 +306,7 @@ void DirTreeView::rowsRemoved(const QModelIndex& parent, int start, int end) {
 
 void DirTreeView::doQueuedDeletions() {
     if(!queuedForDeletion_.empty()) {
-        Q_FOREACH(DirTreeModelItem* item, queuedForDeletion_) {
+        for(DirTreeModelItem* const item : qAsConst(queuedForDeletion_)) {
             delete item;
         }
         queuedForDeletion_.clear();
