@@ -36,6 +36,9 @@
 
 #include "core/archiver.h"
 
+#include "core/legacy/fm-app-info.h"
+
+
 namespace Fm {
 
 FileMenu::FileMenu(Fm::FileInfoList files, std::shared_ptr<const Fm::FileInfo> info, Fm::FilePath cwd, bool isWritableDir, const QString& title, QWidget* parent):
@@ -118,7 +121,7 @@ FileMenu::FileMenu(Fm::FileInfoList files, std::shared_ptr<const Fm::FileInfo> i
 
     createAction_ = new QAction(tr("Create &New"), this);
     Fm::FilePath dirPath = files_.size() == 1 && info_->isDir() ? path : cwd_;
-    createAction_->setMenu(new CreateNewMenu(nullptr, dirPath, this));
+    createAction_->setMenu(new CreateNewMenu(parent, dirPath, this));
     addAction(createAction_);
 
     separator2_ = addSeparator();
@@ -205,32 +208,30 @@ FileMenu::FileMenu(Fm::FileInfoList files, std::shared_ptr<const Fm::FileInfo> i
     // archiver integration
     // FIXME: we need to modify upstream libfm to include some Qt-based archiver programs.
     if(!allVirtual_) {
-        if(sameType_) {
-            auto archiver = Archiver::defaultArchiver();
-            if(archiver) {
-                if(archiver->isMimeTypeSupported(mime_type->name())) {
-                    QAction* archiverSeparator = nullptr;
-                    if(cwd_ && archiver->canExtractArchivesTo()) {
-                        archiverSeparator = addSeparator();
-                        QAction* action = new QAction(tr("Extract to..."), this);
-                        connect(action, &QAction::triggered, this, &FileMenu::onExtract);
-                        addAction(action);
-                    }
-                    if(archiver->canExtractArchives()) {
-                        if(!archiverSeparator) {
-                            addSeparator();
-                        }
-                        QAction* action = new QAction(tr("Extract Here"), this);
-                        connect(action, &QAction::triggered, this, &FileMenu::onExtractHere);
-                        addAction(action);
-                    }
-                }
-                else if(archiver->canCreateArchive()){
-                    addSeparator();
-                    QAction* action = new QAction(tr("Compress"), this);
-                    connect(action, &QAction::triggered, this, &FileMenu::onCompress);
+        auto archiver = Archiver::defaultArchiver();
+        if(archiver) {
+            if(sameType_ && archiver->isMimeTypeSupported(mime_type->name())) {
+                QAction* archiverSeparator = nullptr;
+                if(cwd_ && archiver->canExtractArchivesTo()) {
+                    archiverSeparator = addSeparator();
+                    QAction* action = new QAction(tr("Extract to..."), this);
+                    connect(action, &QAction::triggered, this, &FileMenu::onExtract);
                     addAction(action);
                 }
+                if(archiver->canExtractArchives()) {
+                    if(!archiverSeparator) {
+                        addSeparator();
+                    }
+                    QAction* action = new QAction(tr("Extract Here"), this);
+                    connect(action, &QAction::triggered, this, &FileMenu::onExtractHere);
+                    addAction(action);
+                }
+            }
+            else if(archiver->canCreateArchive()){
+                addSeparator();
+                QAction* action = new QAction(tr("Compress"), this);
+                connect(action, &QAction::triggered, this, &FileMenu::onCompress);
+                addAction(action);
             }
         }
     }
@@ -245,6 +246,18 @@ FileMenu::FileMenu(Fm::FileInfoList files, std::shared_ptr<const Fm::FileInfo> i
 FileMenu::~FileMenu() {
 }
 
+void FileMenu::addTrustAction() {
+    if(info_->isExecutableType()) {
+        QAction* trustAction = new QAction(files_.size() > 1
+                                             ? tr("Trust selected executables")
+                                             : tr("Trust this executable"),
+                                           this);
+        trustAction->setCheckable(true);
+        trustAction->setChecked(info_->isTrustable());
+        connect(trustAction, &QAction::toggled, this, &FileMenu::onTrustToggled);
+        insertAction(propertiesAction_, trustAction);
+    }
+}
 
 void FileMenu::addCustomActionItem(QMenu* menu, std::shared_ptr<const FileActionItem> item) {
     if(!item) { // separator
@@ -307,6 +320,7 @@ void FileMenu::openFilesWithApp(GAppInfo* app) {
         auto uri = file->path().uri();
         uris = g_list_prepend(uris, uri.release());
     }
+    uris = g_list_reverse(uris); // respect the original order
     fm_app_info_launch_uris(app, uris, nullptr, nullptr);
     g_list_foreach(uris, (GFunc)g_free, nullptr);
     g_list_free(uris);
@@ -329,6 +343,12 @@ void FileMenu::onCustomActionTrigerred() {
     }
 }
 
+void FileMenu::onTrustToggled(bool checked) {
+    for(auto& file: files_) {
+        file->setTrustable(checked);
+    }
+}
+
 void FileMenu::onFilePropertiesTriggered() {
     FilePropsDialog::showForFiles(files_);
 }
@@ -344,15 +364,15 @@ void FileMenu::onCutTriggered() {
 void FileMenu::onDeleteTriggered() {
     auto paths = files_.paths();
     if(useTrash_) {
-        FileOperation::trashFiles(paths, confirmTrash_);
+        FileOperation::trashFiles(paths, confirmTrash_, parentWidget());
     }
     else {
-        FileOperation::deleteFiles(paths, confirmDelete_);
+        FileOperation::deleteFiles(paths, confirmDelete_, parentWidget());
     }
 }
 
 void FileMenu::onUnTrashTriggered() {
-    FileOperation::unTrashFiles(files_.paths());
+    FileOperation::unTrashFiles(files_.paths(), parentWidget());
 }
 
 void FileMenu::onPasteTriggered() {
