@@ -23,10 +23,10 @@
 #include "renamedialog.h"
 #include <QLabel>
 #include <QMessageBox>
-#include <libfm/fm-config.h>
 #include "utilities.h"
 #include "ui_file-operation-dialog.h"
 
+#include "core/legacy/fm-config.h"
 
 namespace Fm {
 
