@@ -33,6 +33,8 @@
 #include "core/totalsizejob.h"
 #include "core/folder.h"
 
+#include "core/legacy/fm-config.h"
+
 #define DIFFERENT_UIDS    ((uid)-1)
 #define DIFFERENT_GIDS    ((gid)-1)
 #define DIFFERENT_PERMS   ((mode_t)-1)
@@ -66,6 +68,11 @@ FilePropsDialog::FilePropsDialog(Fm::FileInfoList files, QWidget* parent, Qt::Wi
 
     initGeneralPage();
     initPermissionsPage();
+
+    if(!singleFile || !hasDir) { // not a single dir
+        ui->contentsLabel->hide();
+        ui->fileNumber->hide();
+    }
 }
 
 FilePropsDialog::~FilePropsDialog() {
@@ -314,6 +321,29 @@ void FilePropsDialog::initGeneralPage() {
     connect(totalSizeJob, &Fm::TotalSizeJob::finished, this, &FilePropsDialog::onDeepCountJobFinished, Qt::BlockingQueuedConnection);
     totalSizeJob->setAutoDelete(true);
     totalSizeJob->runAsync();
+
+    // disk usage
+    bool canShowDeviceUsage = false;
+    if(fileInfo->dirPath()) { // skip directories like "search:///"
+        auto folder = Fm::Folder::fromPath(fileInfo->dirPath());
+        if(!folder->isLoaded() && fileInfo->isDir()) { // an empty space is right clicked
+            folder = Fm::Folder::fromPath(fileInfo->path());
+        }
+        guint64 free, total;
+        if(folder->getFilesystemInfo(&total, &free)) {
+            canShowDeviceUsage = true;
+            ui->progressBar->setValue(qRound(static_cast<qreal>((total - free) * 100) / static_cast<qreal>(total)));
+            ui->progressBar->setFormat(tr("%p% used"));
+            ui->spaceLabel->setText(tr("%1 Free of %2")
+                                    .arg(formatFileSize(free, false))
+                                    .arg(formatFileSize(total, false)));
+        }
+    }
+    if(!canShowDeviceUsage) {
+        ui->deviceLabel->setVisible(false);
+        ui->spaceLabel->setVisible(false);
+        ui->progressBar->setVisible(false);
+    }
 }
 
 void FilePropsDialog::onDeepCountJobFinished() {
@@ -345,6 +375,17 @@ void FilePropsDialog::onFileSizeTimerTimeout() {
               QString(" (%1 B)").arg(totalSizeJob->totalOnDiskSize());
         // tr(" (%n) byte(s)", "", deepCountJob->total_ondisk_size);
         ui->onDiskSize->setText(str);
+
+        if(ui->contentsLabel->isVisible()) {
+            unsigned int fc =  totalSizeJob->fileCount(); // the directory is included
+            if (fc <= 1)
+                str = tr("no file");
+            else if (fc == 2)
+                str = tr("one file");
+            else
+                str = tr("%Ln files", "", fc - 1);
+            ui->fileNumber->setText(str);
+        }
     }
 }
 
@@ -385,7 +426,7 @@ void FilePropsDialog::onIconButtonclicked() {
             QString iconName = parts.at(parts.count() - 1);
             int ln = iconName.lastIndexOf(".");
             if(ln > -1) {
-                iconName.remove(ln, iconName.length() - ln); 
+                iconName.remove(ln, iconName.length() - ln);
                 customIcon = QIcon::fromTheme(iconName);
                 ui->iconButton->setIcon(customIcon);
             }
