@@ -33,7 +33,7 @@ FileSearchDialog::FileSearchDialog(QStringList paths, QWidget* parent, Qt::Windo
     ui->setupUi(this);
     ui->minSize->setMaximum(std::numeric_limits<int>().max());
     ui->maxSize->setMaximum(std::numeric_limits<int>().max());
-    Q_FOREACH(const QString& path, paths) {
+    for(const QString& path : qAsConst(paths)) {
         ui->listView->addItem(path);
     }
 
@@ -120,7 +120,7 @@ void FileSearchDialog::accept() {
             fm_search_set_min_mtime(search, ui->minTime->date().toString(QStringLiteral("yyyy-MM-dd")).toUtf8().constData());
         }
 
-        searchUri_ = Path::wrapPtr(fm_search_dup_path(search));
+        searchUri_ = FilePath{fm_search_to_gfile(search), false};
 
         fm_search_free(search);
     }
@@ -144,7 +144,8 @@ void FileSearchDialog::onAddPath() {
 
 void FileSearchDialog::onRemovePath() {
     // remove selected items
-    Q_FOREACH(QListWidgetItem* item, ui->listView->selectedItems()) {
+    const auto itemList = ui->listView->selectedItems();
+    for(QListWidgetItem* const item : itemList) {
         delete item;
     }
 }
