@@ -39,9 +39,13 @@ namespace Fm {
 FolderItemDelegate::FolderItemDelegate(QAbstractItemView* view, QObject* parent):
     QStyledItemDelegate(parent ? parent : view),
     symlinkIcon_(QIcon::fromTheme("emblem-symbolic-link")),
+    untrustedIcon_(QIcon::fromTheme("emblem-important")),
+    addIcon_(QIcon::fromTheme("list-add")),
+    removeIcon_(QIcon::fromTheme("list-remove")),
     fileInfoRole_(Fm::FolderModel::FileInfoRole),
     iconInfoRole_(-1),
     margins_(QSize(3, 3)),
+    shadowHidden_(false),
     hasEditor_(false) {
     connect(this,  &QAbstractItemDelegate::closeEditor, [=]{hasEditor_ = false;});
 }
@@ -76,7 +80,7 @@ QSize FolderItemDelegate::sizeHint(const QStyleOptionViewItem& option, const QMo
         return itemSize_;
     }
 
-    // The default size hint of the horizontal layout isn't reliable 
+    // The default size hint of the horizontal layout isn't reliable
     // because Qt calculates the row size based on the real icon size,
     // which may not be equal to the requested icon size on various occasions.
     // So, we do as in QStyledItemDelegate::sizeHint() but use the requested size.
@@ -115,13 +119,18 @@ void FolderItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem& op
     initStyleOption(&opt, index);
 
     // distinguish the hidden items visually by making their texts italic
+    bool shadowIcon(false);
     if(file && file->isHidden()) {
         QFont f(opt.font);
         f.setItalic(true);
         opt.font = f;
+        shadowIcon = shadowHidden_;
     }
 
     bool isSymlink = file && file->isSymlink();
+    bool isCut = index.data(FolderModel::FileIsCutRole).toBool();
+    // for practical reasons, an emblem is added only to an untrusted, deletable desktop file
+    bool untrusted = file && !file->isTrustable() && file->isDesktopEntry() && file->isDeletable();
     // vertical layout (icon mode, thumbnail mode)
     if(option.decorationPosition == QStyleOptionViewItem::Top ||
             option.decorationPosition == QStyleOptionViewItem::Bottom) {
@@ -132,12 +141,11 @@ void FolderItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem& op
         opt.displayAlignment = Qt::AlignTop | Qt::AlignHCenter;
 
         // draw the icon
-        QIcon::Mode iconMode = iconModeFromState(opt.state);
+        QIcon::Mode iconMode = shadowIcon ? QIcon::Disabled : iconModeFromState(opt.state);
         QPoint iconPos(opt.rect.x() + (opt.rect.width() - option.decorationSize.width()) / 2, opt.rect.y() + margins_.height());
         QPixmap pixmap = opt.icon.pixmap(option.decorationSize, iconMode);
         // in case the pixmap is smaller than the requested size
         QSize margin = ((option.decorationSize - pixmap.size()) / 2).expandedTo(QSize(0, 0));
-        bool isCut = index.data(FolderModel::FileIsCutRole).toBool();
         if(isCut) {
             painter->save();
             painter->setOpacity(0.45);
@@ -153,6 +161,11 @@ void FolderItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem& op
             painter->drawPixmap(iconPos, symlinkIcon_.pixmap(option.decorationSize / 2, iconMode));
         }
 
+        if(untrusted) {
+            // emblem for untrusted, deletable desktop files
+            painter->drawPixmap(iconPos.x(), opt.rect.y() + option.decorationSize.height() / 2, untrustedIcon_.pixmap(option.decorationSize / 2, iconMode));
+        }
+
         // draw other emblems if there's any
         if(!emblems.empty()) {
             // FIXME: we only support one emblem now
@@ -161,6 +174,36 @@ void FolderItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem& op
             painter->drawPixmap(emblemPos, emblem.pixmap(option.decorationSize / 2, iconMode));
         }
 
+        // Draw select/deselect icons outside the main icon but near its top left corner,
+        // with its 1/3 size and only if the icon size isn't smaller than 48 px
+        // (otherwise, the user could not click on them easily).
+        if(option.decorationSize.width() >= 48 && opt.state & QStyle::State_MouseOver) {
+            int s = option.decorationSize.width() / 3;
+            bool cursorOnSelectionCorner = false;
+            iconPos = QPoint(qMax(opt.rect.x(), iconPos.x() - s),
+                             qMax(opt.rect.y(), iconPos.y() - s));
+            if(const QAbstractItemView* iv = qobject_cast<const QAbstractItemView*>(opt.widget)) {
+                QPoint curPos = iv->viewport()->mapFromGlobal(QCursor::pos());
+                if(curPos.x() >= iconPos.x() && curPos.x() <= iconPos.x() + s
+                   && curPos.y() >= iconPos.y() && curPos.y() <= iconPos.y() + s) {
+                    cursorOnSelectionCorner = true;
+                }
+            }
+            if(!cursorOnSelectionCorner) { // make it translucent when not under the cursor
+                painter->save();
+                painter->setOpacity(0.6);
+            }
+            if(opt.state & QStyle::State_Selected) {
+                painter->drawPixmap(iconPos, removeIcon_.pixmap(QSize(s, s), QIcon::Normal));
+            }
+            else {
+                painter->drawPixmap(iconPos, addIcon_.pixmap(QSize(s, s), QIcon::Normal));
+            }
+            if(!cursorOnSelectionCorner) {
+                painter->restore();
+            }
+        }
+
         // draw the text
         QSize drawAreaSize = itemSize_ - 2 * margins_;
         // The text rect dimensions should be exactly as they were in sizeHint()
@@ -173,26 +216,56 @@ void FolderItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem& op
     }
     else {  // horizontal layout (list view)
 
-        // let QStyledItemDelegate does its default painting
-        // FIXME: For better text alignment, here we should increase
-        // the icon width if it's smaller that the requested size
-        QStyledItemDelegate::paint(painter, opt, index);
+        // Let the style engine do the painting but take care of shadowed and cut icons.
+        // NOTE: The shadowing can also be done directly.
+        // WARNING: QStyledItemDelegate shouldn't be used for painting because it resets the icon.
+        // FIXME: For better text alignment, here we could have increased the icon width
+        // when it's smaller than the requested size.
 
-        // draw emblems if needed
-        if(isSymlink || !emblems.empty()) {
-            QIcon::Mode iconMode = iconModeFromState(opt.state);
-            // draw some emblems for the item if needed
-            if(isSymlink) {
-                QPoint iconPos(opt.rect.x(), opt.rect.y() + (opt.rect.height() - option.decorationSize.height()) / 2);
-                painter->drawPixmap(iconPos, symlinkIcon_.pixmap(option.decorationSize / 2, iconMode));
+        QIcon::Mode iconMode = shadowIcon ? QIcon::Disabled : iconModeFromState(opt.state);
+
+        if(!opt.icon.isNull()) {
+            if(shadowIcon) {
+                QPixmap pixmap = opt.icon.pixmap(option.decorationSize, iconMode);
+                opt.icon = QIcon(pixmap);
             }
-            else {
-                // FIXME: we only support one emblem now
-                QPoint iconPos(opt.rect.x() + option.decorationSize.width() / 2, opt.rect.y() + opt.rect.height() / 2);
-                QIcon emblem = emblems.front()->qicon();
-                painter->drawPixmap(iconPos, emblem.pixmap(option.decorationSize / 2, iconMode));
+
+            if(isCut) {
+                QPixmap pixmap = opt.icon.pixmap(option.decorationSize, iconMode);
+                QImage img = pixmap.toImage().convertToFormat(QImage::Format_ARGB32_Premultiplied);
+                img.fill(Qt::transparent);
+                QPainter p(&img);
+                p.setOpacity(0.45);
+                p.drawPixmap(0, 0, pixmap);
+                p.end();
+                opt.icon = QIcon(QPixmap::fromImage(img));
             }
         }
+
+        const QWidget* widget = opt.widget;
+        QStyle* style = widget ? widget->style() : QApplication::style();
+        style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);
+
+        if(isCut && !opt.icon.isNull()) {
+            // restore the original icon to not make it translucent more than once
+            opt.icon = option.icon;
+        }
+
+        // draw some emblems for the item if needed
+        if(isSymlink) {
+            QPoint iconPos(opt.rect.x(), opt.rect.y() + (opt.rect.height() - option.decorationSize.height()) / 2);
+            painter->drawPixmap(iconPos, symlinkIcon_.pixmap(option.decorationSize / 2, iconMode));
+        }
+        if(untrusted) {
+            QPoint iconPos(opt.rect.x(), opt.rect.y() + opt.rect.height() / 2);
+            painter->drawPixmap(iconPos, untrustedIcon_.pixmap(option.decorationSize / 2, iconMode));
+        }
+        if(!emblems.empty()) {
+            // FIXME: we only support one emblem now
+            QPoint iconPos(opt.rect.x() + option.decorationSize.width() / 2, opt.rect.y() + opt.rect.height() / 2);
+            QIcon emblem = emblems.front()->qicon();
+            painter->drawPixmap(iconPos, emblem.pixmap(option.decorationSize / 2, iconMode));
+        }
     }
 }
 
@@ -361,7 +434,8 @@ QWidget* FolderItemDelegate::createEditor(QWidget* parent, const QStyleOptionVie
         // ensure that its background isn't transparent (on the side-pane)
         QWidget* editor = QStyledItemDelegate::createEditor(parent, option, index);
         QPalette p = editor->palette();
-        p.setColor(QPalette::Base, QApplication::palette().color(QPalette::Base));
+        p.setColor(QPalette::Text, qApp->palette().text().color());
+        p.setColor(QPalette::Base, qApp->palette().color(QPalette::Base));
         editor->setPalette(p);
         return editor;
     }
