@@ -35,7 +35,9 @@
 namespace Fm {
 
 FolderModel::FolderModel():
-    hasPendingThumbnailHandler_{false} {
+    hasPendingThumbnailHandler_{false},
+    showFullNames_{false},
+    isLoaded_{false} {
 }
 
 FolderModel::~FolderModel() {
@@ -58,17 +60,20 @@ void FolderModel::setFolder(const std::shared_ptr<Fm::Folder>& new_folder) {
         connect(folder_.get(), &Fm::Folder::filesRemoved, this, &FolderModel::onFilesRemoved);
         // handle the case if the folder is already loaded
         if(folder_->isLoaded()) {
+            isLoaded_ = true;
             insertFiles(0, folder_->files());
         }
     }
 }
 
 void FolderModel::onStartLoading() {
+    isLoaded_ = false;
     // remove all items
     removeAll();
 }
 
 void FolderModel::onFinishLoading() {
+    isLoaded_ = true;
 }
 
 void FolderModel::onFilesAdded(const Fm::FileInfoList& files) {
@@ -85,7 +90,10 @@ void FolderModel::onFilesAdded(const Fm::FileInfoList& files) {
         items.append(item);
     }
     endInsertRows();
-    Q_EMIT filesAdded(files);
+
+    if(isLoaded_) {
+        Q_EMIT filesAdded(files);
+    }
 }
 
 void FolderModel::onFilesChanged(std::vector<Fm::FileInfoPair>& files) {
@@ -221,7 +229,8 @@ QVariant FolderModel::data(const QModelIndex& index, int role/* = Qt::DisplayRol
     case Qt::DisplayRole:  {
         switch(index.column()) {
         case ColumnFileName:
-            return item->displayName();
+            return (showFullNames_ && !item->name().empty() ? QString::fromStdString(item->name())
+                                                            : item->displayName());
         case ColumnFileType:
             return QString(info->mimeType()->desc());
         case ColumnFileMTime:
