@@ -37,19 +37,20 @@
 #include <QMimeData>
 #include <QHoverEvent>
 #include <QApplication>
+#include <QPainter>
 #include <QScrollBar>
 #include <QMetaType>
 #include <QMessageBox>
 #include <QLineEdit>
 #include <QTextEdit>
+#include <QWidgetAction> // for detailed list header context menu
+#include <QLabel> // for detailed list header context menu
 #include <QX11Info> // for XDS support
 #include <xcb/xcb.h> // for XDS support
 #include "xdndworkaround.h" // for XDS support
 #include "folderview_p.h"
 #include "utilities.h"
 
-Q_DECLARE_OPAQUE_POINTER(FmFileInfo*)
-
 #define SCROLL_FRAMES_PER_SEC 50
 #define SCROLL_DURATION 300 // in ms
 
@@ -59,10 +60,12 @@ using namespace Fm;
 
 FolderViewListView::FolderViewListView(QWidget* parent):
     QListView(parent),
-    activationAllowed_(true) {
+    activationAllowed_(true),
+    cursorOnSelectionCorner_(false) {
     connect(this, &QListView::activated, this, &FolderViewListView::activation);
     // inline renaming
     setEditTriggers(QAbstractItemView::NoEditTriggers);
+    setMouseTracking(true); // needed with selection corner icon
 }
 
 FolderViewListView::~FolderViewListView() {
@@ -78,6 +81,9 @@ void FolderViewListView::startDrag(Qt::DropActions supportedActions) {
 }
 
 void FolderViewListView::mousePressEvent(QMouseEvent* event) {
+    setSelectionMode(cursorOnSelectionCorner_ && event->button() == Qt::LeftButton
+                         ? QAbstractItemView::MultiSelection
+                         : QAbstractItemView::ExtendedSelection);
     QListView::mousePressEvent(event);
     static_cast<FolderView*>(parent())->childMousePressEvent(event);
 }
@@ -85,12 +91,22 @@ void FolderViewListView::mousePressEvent(QMouseEvent* event) {
 void FolderViewListView::mouseMoveEvent(QMouseEvent* event) {
     // NOTE: Filter the BACK & FORWARD buttons to not Drag & Drop with them.
     // (by default Qt views drag with any button)
-    if (event->buttons() == Qt::NoButton || event->buttons() & ~(Qt::BackButton | Qt::ForwardButton))
+    if (event->buttons() == Qt::NoButton || (event->buttons() & ~(Qt::BackButton | Qt::ForwardButton))) {
+        bool cursorOnSelectionCorner = cursorOnSelectionCorner_;
         QListView::mouseMoveEvent(event);
+        // update the index if the cursor enters/leaves the selection corner icon
+        if(cursorOnSelectionCorner != cursorOnSelectionCorner_ && event->buttons() == Qt::NoButton) {
+            update(indexAt(event->pos()));
+        }
+    }
 }
 
 QModelIndex FolderViewListView::indexAt(const QPoint& point) const {
     QModelIndex index = QListView::indexAt(point);
+    bool isCursorPos(point == viewport()->mapFromGlobal(QCursor::pos()));
+    if(isCursorPos) {
+        cursorOnSelectionCorner_ = false;
+    }
     // NOTE: QListView has a severe design flaw here. It does hit-testing based on the
     // total bound rect of the item. The width of an item is determined by max(icon_width, text_width).
     // So if the text label is much wider than the icon, when you click outside the icon but
@@ -103,12 +119,27 @@ QModelIndex FolderViewListView::indexAt(const QPoint& point) const {
         FolderItemDelegate* delegate = static_cast<FolderItemDelegate*>(itemDelegateForColumn(FolderModel::ColumnFileName));
         QSize margins = delegate->getMargins();
         QSize _iconSize = iconSize();
-        if(point.y() < visRect.top() + margins.height()) { // above icon
+        int iconXMargin = (visRect.width() - _iconSize.width()) / 2;
+        int iconLeft = visRect.left() + iconXMargin;
+        int iconTop = visRect.top() + margins.height();
+        // the selection (hover) corner is a rectangle near the top left corner of
+        // the icon and outside it as far as possible, so that its width and height
+        // are 1/3 of the icon size >= 48 px (see FolderItemDelegate::paint)
+        if(isCursorPos && _iconSize.width() >= 48) {
+            int s = _iconSize.width() / 3;
+            iconLeft = qMax(visRect.left(), iconLeft - s);
+            iconTop = qMax(visRect.top(), iconTop - s);
+            if(point.x() >= iconLeft &&  point.x() <= iconLeft + s
+               && point.y() >= iconTop &&  point.y() <= iconTop + s) {
+                cursorOnSelectionCorner_ = true;
+                return index;
+            }
+        }
+        if(point.y() < iconTop) { // above icon
             return QModelIndex();
         }
         else if(point.y() < visRect.top() + margins.height() + _iconSize.height()) { // on the icon area
-            int iconXMargin = (visRect.width() - _iconSize.width()) / 2;
-            if(point.x() < (visRect.left() + iconXMargin) || point.x() > (visRect.right() + 1 - iconXMargin)) {
+            if(point.x() < iconLeft || point.x() > (visRect.right() + 1 - iconXMargin)) {
                 // to the left or right of the icon
                 return QModelIndex();
             }
@@ -186,7 +217,10 @@ void FolderViewListView::dropEvent(QDropEvent* e) {
 
 void FolderViewListView::mouseReleaseEvent(QMouseEvent* event) {
     bool activationWasAllowed = activationAllowed_;
-    if((!style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, nullptr, this)) || (event->button() != Qt::LeftButton)) {
+    if(!style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, nullptr, this)
+       || event->button() != Qt::LeftButton
+       // no activation with mouse when the cursor is on the selection corner
+       || cursorOnSelectionCorner_) {
         activationAllowed_ = false;
     }
 
@@ -197,7 +231,10 @@ void FolderViewListView::mouseReleaseEvent(QMouseEvent* event) {
 
 void FolderViewListView::mouseDoubleClickEvent(QMouseEvent* event) {
     bool activationWasAllowed = activationAllowed_;
-    if((style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, nullptr, this)) || (event->button() != Qt::LeftButton)) {
+    if(style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, nullptr, this)
+       || event->button() != Qt::LeftButton
+       // no activation with mouse when the cursor is on the selection corner
+       || cursorOnSelectionCorner_) {
         activationAllowed_ = false;
     }
 
@@ -238,13 +275,31 @@ FolderViewTreeView::FolderViewTreeView(QWidget* parent):
     QTreeView(parent),
     doingLayout_(false),
     layoutTimer_(nullptr),
-    activationAllowed_(true) {
+    activationAllowed_(true),
+    ctrlDragSelectionFlag_(QItemSelectionModel::NoUpdate) {
 
+    header()->setSectionResizeMode(QHeaderView::Interactive);
     header()->setStretchLastSection(true);
+
+    // get the new width if the section is resized by user
+    connect(header(), &QHeaderView::sectionResized, [this](int logicalIndex, int/* oldSize*/, int newSize) {
+        if(doingLayout_ || customColumnWidths_.isEmpty()) {
+            return;
+        }
+        int vIndx = header()->visualIndex(logicalIndex);
+        if(vIndx >= 0 && vIndx < customColumnWidths_.size()) {
+            customColumnWidths_[vIndx] = newSize;
+            Q_EMIT columnResizedByUser(vIndx, newSize);
+            queueLayoutColumns();
+        }
+    });
+
+    // header context menu for configuring its resizing and hidden sections
+    header()->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(header(), &QWidget::customContextMenuRequested, this, &FolderViewTreeView::headerContextMenu);
+
     setIndentation(0);
-    /* the default true value may cause a crash on entering a folder
-       by double clicking because of the viewport update done by
-       QTreeView::mouseDoubleClickEvent() (a Qt bug?) */
+    // the default true value may cause a crash on entering a folder by double clicking (a Qt bug?)
     setExpandsOnDoubleClick(false);
 
     connect(this, &QTreeView::activated, this, &FolderViewTreeView::activation);
@@ -258,6 +313,82 @@ FolderViewTreeView::~FolderViewTreeView() {
     }
 }
 
+void FolderViewTreeView::setCustomColumnWidths(const QList<int> &widths) {
+    // enables cutomizable widths if "widths" is not empty; otherwise, enables auto-resizing
+    if(customColumnWidths_ == widths) {
+        return;
+    }
+    customColumnWidths_.clear();
+    customColumnWidths_ = widths;
+    header()->setStretchLastSection(widths.isEmpty());
+    queueLayoutColumns();
+    if(widths.isEmpty()) {
+        Q_EMIT autoResizeEnabled();
+    }
+}
+
+void FolderViewTreeView::setHiddenColumns(const QSet<int> &columns) {
+    if(hiddenColumns_ == columns) {
+        return;
+    }
+    hiddenColumns_.clear();
+    hiddenColumns_ = columns;
+    queueLayoutColumns();
+}
+
+void FolderViewTreeView::headerContextMenu(const QPoint &p) {
+    QMenu menu;
+    QAction *action = menu.addAction (tr("Auto-resize columns"));
+    action->setCheckable(true);
+    action->setChecked(customColumnWidths_.isEmpty());
+    connect(action, &QAction::triggered, action, [this] (bool checked) {
+        QList<int> widths;
+        if(!checked) {
+            for(int column = 0; column < FolderModel::NumOfColumns; ++column) {
+                widths << 0;
+            }
+            // one signal is enough to make a raw FolderView::customColumnWidths_
+            Q_EMIT columnResizedByUser(0, 0);
+        }
+        setCustomColumnWidths(widths);
+    });
+    if(model()) {
+        menu.addSeparator();
+        QWidgetAction *labelAction = new QWidgetAction(&menu);
+        QLabel *label = new QLabel("<center><b>" + tr("Visible Columns") + "</b></center>");
+        labelAction->setDefaultWidget(label);
+        menu.addAction (labelAction);
+
+        int filenameColumn = header()->visualIndex(FolderModel::ColumnFileName);
+        int numCols = header()->count();
+        for(int column = 0; column < numCols; ++column) {
+            int columnId = header()->logicalIndex(column);
+            if(columnId >= 0 && columnId < FolderModel::NumOfColumns) {
+                action = menu.addAction (model()->headerData(columnId, Qt::Horizontal, Qt::DisplayRole).toString());
+                action->setCheckable(true);
+                if(columnId == filenameColumn) { // never hide the name column
+                    action->setChecked(true);
+                    action->setDisabled(true);
+                }
+                else {
+                    action->setChecked(!header()->isSectionHidden(columnId));
+                    connect(action, &QAction::triggered, action, [this, column, columnId] (bool checked) {
+                        if(checked) {
+                            hiddenColumns_.remove(column);
+                        }
+                        else {
+                            hiddenColumns_ << column;
+                        }
+                        Q_EMIT columnHiddenByUser(column, !checked);
+                        queueLayoutColumns();
+                    });
+                }
+            }
+        }
+    }
+    menu.exec(header()->mapToGlobal(p));
+}
+
 void FolderViewTreeView::setModel(QAbstractItemModel* model) {
     QTreeView::setModel(model);
     layoutColumns();
@@ -268,16 +399,116 @@ void FolderViewTreeView::setModel(QAbstractItemModel* model) {
     }
 }
 
+void FolderViewTreeView::paintEvent(QPaintEvent * event) {
+    QTreeView::paintEvent(event);
+    if(rubberBandRect_.isValid()) { // draw rubberband
+        QPainter p(viewport());
+        QStyleOptionRubberBand opt;
+        opt.initFrom(this);
+        opt.shape = QRubberBand::Rectangle;
+        opt.opaque = false;
+        QRect r = rubberBandRect_.adjusted(-horizontalOffset(), -verticalOffset(),
+                                            -horizontalOffset(), -verticalOffset())
+                    .intersected(viewport()->rect()
+                    .adjusted(-16, -16, 16, 16));
+        opt.rect = r;
+        style()->drawControl(QStyle::CE_RubberBand, &opt, &p);
+    }
+}
+
 void FolderViewTreeView::mousePressEvent(QMouseEvent* event) {
-    QTreeView::mousePressEvent(event);
+    QAbstractItemView::mousePressEvent(event);
+
+    // remember mouse press position and determine whether selections should be kept
+    // or removed later, when the cursor moves
+    mousePressPoint_ = event->pos() + QPoint(horizontalOffset(), verticalOffset());
+    QModelIndex index = indexAt(event->pos());
+    if(index.isValid()) {
+        QItemSelectionModel::SelectionFlags command;
+        Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+        const Qt::MouseButton button = static_cast<const QMouseEvent*>(event)->button();
+        const bool shiftKeyPressed = modifiers & Qt::ShiftModifier;
+        const bool controlKeyPressed = modifiers & Qt::ControlModifier;
+        const bool rightButtonPressed = button & Qt::RightButton;
+        const bool indexIsSelected = selectionModel()->isSelected(index);
+        if(controlKeyPressed && !shiftKeyPressed && !rightButtonPressed) {
+            ctrlDragSelectionFlag_ = indexIsSelected ? QItemSelectionModel::Deselect : QItemSelectionModel::Select;
+        }
+    }
+
     static_cast<FolderView*>(parent())->childMousePressEvent(event);
 }
 
 void FolderViewTreeView::mouseMoveEvent(QMouseEvent* event) {
     // NOTE: Filter the BACK & FORWARD buttons to not Drag & Drop with them.
     // (by default Qt views drag with any button)
-    if (event->buttons() == Qt::NoButton || event->buttons() & ~(Qt::BackButton | Qt::ForwardButton))
-        QTreeView::mouseMoveEvent(event);
+    if(event->buttons() == Qt::NoButton || (event->buttons() & ~(Qt::BackButton | Qt::ForwardButton))) {
+        // handle rubberband
+        if((event->buttons() & Qt::LeftButton)
+            && (rubberBandRect_.isValid()
+                || !indexAt(mousePressPoint_ - QPoint(horizontalOffset(), verticalOffset())).isValid())) {
+            QAbstractItemView::mouseMoveEvent(event);
+
+            // set rubberband rectangle
+            QRect rect(mousePressPoint_, event->pos() + QPoint(horizontalOffset(), verticalOffset()));
+            rect = rect.normalized();
+            QRect r = rect.united(rubberBandRect_);
+            viewport()->update(r.adjusted(-horizontalOffset(), -verticalOffset(),
+                                            -horizontalOffset(), -verticalOffset()));
+            rubberBandRect_ = rect;
+
+            // set state and selection
+            setState(QAbstractItemView::DragSelectingState);
+            Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+            QItemSelectionModel::SelectionFlags command;
+            if(modifiers & Qt::ControlModifier) {
+                command = QItemSelectionModel::ToggleCurrent;
+            }
+            else if(modifiers & Qt::ShiftModifier) {
+                command = QItemSelectionModel::SelectCurrent;
+            }
+            else {
+                command = QItemSelectionModel::Clear|QItemSelectionModel::SelectCurrent;
+            }
+            command |= QItemSelectionModel::Rows;
+            if(ctrlDragSelectionFlag_ != QItemSelectionModel::NoUpdate && command.testFlag(QItemSelectionModel::Toggle)) {
+                command &= ~QItemSelectionModel::Toggle;
+                command |= ctrlDragSelectionFlag_;
+            }
+            QRect selectionRect = QRect(rubberBandRect_.topLeft(), rubberBandRect_.bottomRight());
+            setSelection(selectionRect, command);
+        }
+        else {
+            QTreeView::mouseMoveEvent(event);
+        }
+    }
+}
+
+void FolderViewTreeView::setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags command) {
+    if(model() && state() == QAbstractItemView::DragSelectingState) { // rubberband selection
+        QRect r = rubberBandRect_.adjusted(-horizontalOffset(), -verticalOffset(),
+                                            -horizontalOffset(), -verticalOffset());
+        r.setLeft(qMax(0, r.left()));
+        r.setTop(qMax(-verticalOffset(), r.top()));
+        QModelIndex top = indexAt(r.topLeft());
+        QItemSelection selection;
+        if(top.isValid()) {
+            top = top.sibling(top.row(), 0);
+             if(top.isValid()) {
+                QModelIndex bottom = indexAt(r.bottomLeft());
+                if(!bottom.isValid()) {
+                    bottom = model()->index(model()->rowCount() - 1, 0);
+                }
+                if(bottom.isValid()) {
+                    selection = QItemSelection(top, bottom);
+                }
+             }
+        }
+        selectionModel()->select(selection, command | QItemSelectionModel::Rows);
+    }
+    else {
+        QTreeView::setSelection(rect, command);
+    }
 }
 
 void FolderViewTreeView::dragEnterEvent(QDragEnterEvent* event) {
@@ -328,42 +559,81 @@ void FolderViewTreeView::layoutColumns() {
         int column;
         for(column = 0; column < numCols; ++column) {
             int columnId = headerView->logicalIndex(column);
-            // get the size that the column needs
-            if(model_) {
-                QVariant data = model_->headerData(columnId, Qt::Horizontal, Qt::DisplayRole);
-                if(data.isValid()) {
-                    opt.text = data.isValid() ? data.toString() : QString();
+
+            // handle hidden columns
+            bool wasHidden = false;
+            if(headerView->isSectionHidden(columnId)) {
+                if(!hiddenColumns_.contains(columnId)) {
+                    headerView->setSectionHidden(columnId, false);
+                    wasHidden = true;
+                }
+                else {
+                    continue;
                 }
             }
-            opt.section = columnId;
-            widths[column] = qMax(sizeHintForColumn(columnId),
-                                  style()->sizeFromContents(QStyle::CT_HeaderSection, &opt, QSize(), headerView).width());
+            else if(hiddenColumns_.contains(columnId)) {
+                headerView->setSectionHidden(columnId, true);
+                continue;
+            }
+
+            if(customColumnWidths_.size() > column) {
+                // see FolderView::setCustomColumnWidths for the meaning of custom width <= 0
+                if(customColumnWidths_.at(column) > 0) {
+                    widths[column] = qMax(customColumnWidths_.at(column), headerView->minimumSectionSize());
+                }
+                else {
+                    if(wasHidden) {
+                        // WARNING: When a section is shown in the interactive mode, Qt gives
+                        // a huge width to it. As a workaround, the width is set to the minimum here.
+                        customColumnWidths_[column] = widths[column] = headerView->minimumSectionSize();
+                    }
+                    else {
+                        customColumnWidths_[column] = widths[column] = headerView->sectionSize(columnId);
+                    }
+                    Q_EMIT columnResizedByUser(column, customColumnWidths_.at(column));
+                }
+            }
+            else {
+                // get the size that the column needs
+                if(model_) {
+                    QVariant data = model_->headerData(columnId, Qt::Horizontal, Qt::DisplayRole);
+                    if(data.isValid()) {
+                        opt.text = data.isValid() ? data.toString() : QString();
+                    }
+                }
+                opt.section = columnId;
+                widths[column] = qMax(sizeHintForColumn(columnId),
+                                    style()->sizeFromContents(QStyle::CT_HeaderSection, &opt, QSize(),
+                                                                headerView).width());
+            }
             // compute the total width needed
             desiredWidth += widths[column];
         }
 
         int filenameColumn = headerView->visualIndex(FolderModel::ColumnFileName);
-        // if the total witdh we want exceeds the available space
-        if(desiredWidth > availWidth) {
-            // Compute the width available for the filename column
-            int filenameAvailWidth = availWidth - desiredWidth + widths[filenameColumn];
+        if(customColumnWidths_.size() <= filenameColumn) { // practically means no custom width
+            // if the total witdh we want exceeds the available space
+            if(desiredWidth > availWidth) {
+                // Compute the width available for the filename column
+                int filenameAvailWidth = availWidth - desiredWidth + widths[filenameColumn];
 
-            // Compute the minimum acceptable width for the filename column
-            int filenameMinWidth = qMin(200, sizeHintForColumn(filenameColumn));
+                // Compute the minimum acceptable width for the filename column
+                int filenameMinWidth = qMin(200, sizeHintForColumn(filenameColumn));
 
-            if(filenameAvailWidth > filenameMinWidth) {
-                // Shrink the filename column to the available width
-                widths[filenameColumn] = filenameAvailWidth;
+                if(filenameAvailWidth > filenameMinWidth) {
+                    // Shrink the filename column to the available width
+                    widths[filenameColumn] = filenameAvailWidth;
+                }
+                else {
+                    // Set the filename column to its minimum width
+                    widths[filenameColumn] = filenameMinWidth;
+                }
             }
             else {
-                // Set the filename column to its minimum width
-                widths[filenameColumn] = filenameMinWidth;
+                // Fill the extra available space with the filename column
+                widths[filenameColumn] += availWidth - desiredWidth;
             }
         }
-        else {
-            // Fill the extra available space with the filename column
-            widths[filenameColumn] += availWidth - desiredWidth;
-        }
 
         // really do the resizing for every column
         for(int column = 0; column < numCols; ++column) {
@@ -377,6 +647,7 @@ void FolderViewTreeView::layoutColumns() {
         delete layoutTimer_;
         layoutTimer_ = nullptr;
     }
+    setUpdatesEnabled(true);
 }
 
 void FolderViewTreeView::resizeEvent(QResizeEvent* event) {
@@ -393,8 +664,9 @@ void FolderViewTreeView::resizeEvent(QResizeEvent* event) {
 }
 
 void FolderViewTreeView::rowsInserted(const QModelIndex& parent, int start, int end) {
-    QTreeView::rowsInserted(parent, start, end);
+    setUpdatesEnabled(false); // prevent header text flickering
     queueLayoutColumns();
+    QTreeView::rowsInserted(parent, start, end);
 }
 
 void FolderViewTreeView::rowsAboutToBeRemoved(const QModelIndex& parent, int start, int end) {
@@ -415,8 +687,9 @@ void FolderViewTreeView::reset() {
     // might not be called. Hence we also have to re-layout the columns when the model is reset.
     // This fixes bug #190
     // https://github.com/lxqt/pcmanfm-qt/issues/190
-    QTreeView::reset();
+    setUpdatesEnabled(false); // prevent header text flickering
     queueLayoutColumns();
+    QTreeView::reset();
 }
 
 void FolderViewTreeView::queueLayoutColumns() {
@@ -436,9 +709,14 @@ void FolderViewTreeView::mouseReleaseEvent(QMouseEvent* event) {
         activationAllowed_ = false;
     }
 
-    QTreeView::mouseReleaseEvent(event);
+    QAbstractItemView::mouseReleaseEvent(event);
+    viewport()->update(rubberBandRect_.adjusted(-horizontalOffset(), -verticalOffset(),
+                                                -horizontalOffset(), -verticalOffset()));
+    rubberBandRect_ = QRect();
+    ctrlDragSelectionFlag_ = QItemSelectionModel::NoUpdate;
 
     activationAllowed_ = activationWasAllowed;
+
 }
 
 void FolderViewTreeView::mouseDoubleClickEvent(QMouseEvent* event) {
@@ -481,6 +759,7 @@ FolderView::FolderView(FolderView::ViewMode _mode, QWidget *parent):
     autoSelectionTimer_(nullptr),
     selChangedTimer_(nullptr),
     itemDelegateMargins_(QSize(3, 3)),
+    shadowHidden_(false),
     smoothScrollTimer_(nullptr),
     wheelEvent_(nullptr) {
 
@@ -508,8 +787,38 @@ FolderView::~FolderView() {
     }
 }
 
+void FolderView::setCustomColumnWidths(const QList<int> &widths) {
+    customColumnWidths_.clear();
+    customColumnWidths_ = widths;
+    // Complete the widths list with zeros if needed. A value of <= 0 means that
+    // the initial custom width of the column should be set to its current width.
+    if(!customColumnWidths_.isEmpty()) {
+        for(int i = customColumnWidths_.size(); i < FolderModel::NumOfColumns; ++i) {
+            customColumnWidths_ << 0;
+        }
+    }
+    // resize header sections to custom widths if the tree view exists
+    if(mode == DetailedListMode) {
+        if(FolderViewTreeView* treeView = static_cast<FolderViewTreeView*>(view)) {
+            treeView->setCustomColumnWidths(customColumnWidths_);
+        }
+    }
+}
+
+void FolderView::setHiddenColumns(const QList<int> &columns) {
+    hiddenColumns_.clear();
+    hiddenColumns_ = columns.toSet();
+    if(mode == DetailedListMode) {
+        if(FolderViewTreeView* treeView = static_cast<FolderViewTreeView*>(view)) {
+            treeView->setHiddenColumns(hiddenColumns_);
+        }
+    }
+}
+
 void FolderView::onItemActivated(QModelIndex index) {
-    if(index.isValid() && index.model()) {
+    QItemSelectionModel* selModel = selectionModel();
+    if(index.isValid() && index.model()
+       && selModel && selModel->isSelected(index)) { // do nothing when the item is not selected
         QVariant data = index.model()->data(index, FolderModel::FileInfoRole);
         auto info = data.value<std::shared_ptr<const Fm::FileInfo>>();
         if(info) {
@@ -601,7 +910,39 @@ void FolderView::setViewMode(ViewMode _mode) {
     FolderItemDelegate* delegate = nullptr;
     if(mode == DetailedListMode) {
         FolderViewTreeView* treeView = new FolderViewTreeView(this);
+        treeView->setCustomColumnWidths(customColumnWidths_);
+        treeView->setHiddenColumns(hiddenColumns_);
         connect(treeView, &FolderViewTreeView::activatedFiltered, this, &FolderView::onItemActivated);
+        // update the list of custom widhts when the user changes it
+        connect(treeView, &FolderViewTreeView::columnResizedByUser, [this](int visualIndex, int newWidth) {
+            if(visualIndex >= 0) {
+                if(visualIndex < customColumnWidths_.size()){
+                    customColumnWidths_[visualIndex] = newWidth;
+                }
+                else {
+                    customColumnWidths_ << newWidth;
+                }
+                // complete the widths list with zeros if needed
+                for(int i = customColumnWidths_.size(); i < FolderModel::NumOfColumns; ++i) {
+                    customColumnWidths_ << 0;
+                }
+                Q_EMIT columnResizedByUser();
+            }
+        });
+        connect(treeView, &FolderViewTreeView::autoResizeEnabled, [this]() {
+            customColumnWidths_.clear();
+            Q_EMIT columnResizedByUser();
+        });
+        // update the list of hidden columns when the user changes it
+        connect(treeView, &FolderViewTreeView::columnHiddenByUser, [this](int visibleIndex, bool hidden) {
+            if(hidden) {
+                hiddenColumns_ << visibleIndex;
+            }
+            else {
+                hiddenColumns_.remove(visibleIndex);
+            }
+            Q_EMIT columnHiddenByUser();
+        });
         setFocusProxy(treeView);
 
         view = treeView;
@@ -611,6 +952,7 @@ void FolderView::setViewMode(ViewMode _mode) {
 
         // set our own custom delegate
         delegate = new FolderItemDelegate(treeView);
+        delegate->setShadowHidden(shadowHidden_);
         treeView->setItemDelegateForColumn(FolderModel::ColumnFileName, delegate);
     }
     else {
@@ -627,6 +969,7 @@ void FolderView::setViewMode(ViewMode _mode) {
 
         // set our own custom delegate
         delegate = new FolderItemDelegate(listView);
+        delegate->setShadowHidden(shadowHidden_);
         listView->setItemDelegateForColumn(FolderModel::ColumnFileName, delegate);
         // FIXME: should we expose the delegate?
         listView->setMovement(QListView::Static);
@@ -765,6 +1108,24 @@ void FolderView::setMargins(QSize size) {
     }
 }
 
+void FolderView::setShadowHidden(bool shadowHidden) {
+    if(view && shadowHidden != shadowHidden_) {
+        shadowHidden_ = shadowHidden;
+        FolderItemDelegate* delegate = nullptr;
+        if(mode == DetailedListMode) {
+            FolderViewTreeView* treeView = static_cast<FolderViewTreeView*>(view);
+            delegate = static_cast<FolderItemDelegate*>(treeView->itemDelegateForColumn(FolderModel::ColumnFileName));
+        }
+        else {
+            FolderViewListView* listView = static_cast<FolderViewListView*>(view);
+            delegate = static_cast<FolderItemDelegate*>(listView->itemDelegateForColumn(FolderModel::ColumnFileName));
+        }
+        if(delegate) {
+            delegate->setShadowHidden(shadowHidden);
+        }
+    }
+}
+
 FolderView::ViewMode FolderView::viewMode() const {
     return mode;
 }
@@ -909,40 +1270,44 @@ QModelIndex FolderView::indexFromFolderPath(const Fm::FilePath& folderPath) cons
 }
 
 void FolderView::selectFiles(const Fm::FileInfoList& files, bool add) {
-  if(!model_ || files.empty()) {
-      return;
-  }
-  if(!add) {
-      selectionModel()->clear();
-  }
-  QModelIndex index, firstIndex;
-  int count = model_->rowCount();
-  Fm::FileInfoList list = files;
-  bool singleFile(files.size() == 1);
-  for(int row = 0; row < count; ++row) {
-      if (list.empty()) {
-          break;
-      }
-      index = model_->index(row, 0);
-      auto info = model_->fileInfoFromIndex(index);
-      for(auto it = list.cbegin(); it != list.cend(); ++it) {
-          auto& item = *it;
-          if(item == info) {
-              selectionModel()->select(index, QItemSelectionModel::Select);
-              if (!firstIndex.isValid()) {
-                  firstIndex = index;
-              }
-              list.erase(it);
-              break;
-          }
-      }
-  }
-  if (firstIndex.isValid()) {
-      view->scrollTo(firstIndex, QAbstractItemView::EnsureVisible);
-      if (singleFile) { // give focus to the single file
-          selectionModel()->setCurrentIndex(firstIndex, QItemSelectionModel::Current);
-      }
-  }
+    if(!model_ || files.empty()) {
+        return;
+    }
+    if(!add) {
+        selectionModel()->clear();
+    }
+    QModelIndex index, firstIndex;
+    int count = model_->rowCount();
+    Fm::FileInfoList list = files;
+    bool singleFile(files.size() == 1);
+    QItemSelectionModel::SelectionFlags flags = QItemSelectionModel::Select;
+    if(mode == DetailedListMode) {
+        flags |= QItemSelectionModel::Rows;
+    }
+    for(int row = 0; row < count; ++row) {
+        if (list.empty()) {
+            break;
+        }
+        index = model_->index(row, 0);
+        auto info = model_->fileInfoFromIndex(index);
+        for(auto it = list.cbegin(); it != list.cend(); ++it) {
+            auto& item = *it;
+            if(item == info) {
+                selectionModel()->select(index, flags);
+                if (!firstIndex.isValid()) {
+                    firstIndex = index;
+                }
+                list.erase(it);
+                break;
+            }
+        }
+    }
+    if (firstIndex.isValid()) {
+        view->scrollTo(firstIndex, QAbstractItemView::EnsureVisible);
+        if (singleFile) { // give focus to the single file
+            selectionModel()->setCurrentIndex(firstIndex, QItemSelectionModel::Current);
+        }
+    }
 }
 
 Fm::FileInfoList FolderView::selectedFiles() const {
@@ -982,15 +1347,16 @@ void FolderView::selectAll() {
 void FolderView::invertSelection() {
     if(model_) {
         QItemSelectionModel* selModel = view->selectionModel();
-        int rows = model_->rowCount();
-        QItemSelectionModel::SelectionFlags flags = QItemSelectionModel::Toggle;
+        QItemSelectionModel::SelectionFlags flags;
         if(mode == DetailedListMode) {
             flags |= QItemSelectionModel::Rows;
         }
-        for(int row = 0; row < rows; ++row) {
-            QModelIndex index = model_->index(row, 0);
-            selModel->select(index, flags);
-        }
+        // we don't use a "for" loop on rows because it would be slow
+        const QItemSelection _all{model_->index(0, 0), model_->index(model_->rowCount() - 1, 0)};
+        const QItemSelection _old{selModel->selection()};
+
+        selModel->select(_all, QItemSelectionModel::Select);
+        selModel->select(_old, QItemSelectionModel::Deselect);
     }
 }
 
@@ -1231,6 +1597,14 @@ void FolderView::onAutoSelectionTimeout() {
         return;
     }
 
+    // don't do anything if the cursor is on selection corner icon
+    if(mode != DetailedListMode) {
+        FolderViewListView* listView = static_cast<FolderViewListView*>(view);
+        if(listView->cursorOnSelectionCorner()) {
+            return;
+        }
+    }
+
     Qt::KeyboardModifiers mods = QApplication::keyboardModifiers();
     QPoint pos = view->viewport()->mapFromGlobal(QCursor::pos()); // convert to viewport coordinates
     QModelIndex index = view->indexAt(pos); // find out the hovered item
@@ -1318,11 +1692,9 @@ void FolderView::onFileClicked(int type, const std::shared_ptr<const Fm::FileInf
             // show context menu
             auto files = selectedFiles();
             if(!files.empty()) {
-                QModelIndexList selIndexes = mode == DetailedListMode ? selectedRows() : selectedIndexes();
-                Fm::FileMenu* fileMenu = (view && selIndexes.size() == 1)
-                                         ? new Fm::FileMenu(files, fileInfo, folderPath, isWritableDir, QString(), view)
-                                         : new Fm::FileMenu(files, fileInfo, folderPath, isWritableDir);
+                Fm::FileMenu* fileMenu = new Fm::FileMenu(files, fileInfo, folderPath, isWritableDir, QString(), view);
                 fileMenu->setFileLauncher(fileLauncher_);
+                fileMenu->addTrustAction();
                 prepareFileMenu(fileMenu);
                 menu = fileMenu;
             }
