@@ -36,6 +36,8 @@ PlacesModel::PlacesModel(QObject* parent):
     QStandardItemModel(parent),
     showApplications_(true),
     showDesktop_(true),
+    trashMonitor_(nullptr),
+    trashUpdateTimer_(nullptr),
     // FIXME: this seems to be broken when porting to new API.
     ejectIcon_(QIcon::fromTheme("media-eject")) {
     setColumnCount(2);
@@ -168,7 +170,9 @@ PlacesModel::~PlacesModel() {
 
 // static
 void PlacesModel::onTrashChanged(GFileMonitor* /*monitor*/, GFile* /*gf*/, GFile* /*other*/, GFileMonitorEvent /*evt*/, PlacesModel* pThis) {
-    QTimer::singleShot(0, pThis, SLOT(updateTrash()));
+    if(pThis->trashUpdateTimer_ != nullptr && !pThis->trashUpdateTimer_->isActive()) {
+        pThis->trashUpdateTimer_->start(250); // don't update trash very fast
+    }
 }
 
 void PlacesModel::updateTrash() {
@@ -177,7 +181,7 @@ void PlacesModel::updateTrash() {
         QPointer<PlacesModel> model;
         GFile* gf;
         UpdateTrashData(PlacesModel* _model) : model(_model) {
-            gf = fm_file_new_for_uri("trash:///");
+            gf = g_file_new_for_uri("trash:///");
         }
         ~UpdateTrashData() {
             g_object_unref(gf);
@@ -209,7 +213,7 @@ void PlacesModel::updateTrash() {
 
 void PlacesModel::createTrashItem() {
     GFile* gf;
-    gf = fm_file_new_for_uri("trash:///");
+    gf = g_file_new_for_uri("trash:///");
     // check if trash is supported by the current vfs
     // if gvfs is not installed, this can be unavailable.
     if(!g_file_query_exists(gf, nullptr)) {
@@ -220,8 +224,13 @@ void PlacesModel::createTrashItem() {
     }
     trashItem_ = new PlacesModelItem("user-trash", tr("Trash"), Fm::FilePath::fromUri("trash:///"));
 
-    trashMonitor_ = fm_monitor_directory(gf, nullptr);
+    trashMonitor_ = g_file_monitor_directory(gf, G_FILE_MONITOR_NONE, nullptr, nullptr);
     if(trashMonitor_) {
+        if(trashUpdateTimer_ == nullptr) {
+            trashUpdateTimer_ = new QTimer(this);
+            trashUpdateTimer_->setSingleShot(true);
+            connect(trashUpdateTimer_, &QTimer::timeout, this, &PlacesModel::updateTrash);
+        }
         g_signal_connect(trashMonitor_, "changed", G_CALLBACK(onTrashChanged), this);
     }
     g_object_unref(gf);
@@ -250,6 +259,11 @@ void PlacesModel::setShowTrash(bool show) {
     }
     else {
         if(trashItem_) {
+            if(trashUpdateTimer_) {
+                trashUpdateTimer_->stop();
+                delete trashUpdateTimer_;
+                trashUpdateTimer_ = nullptr;
+            }
             if(trashMonitor_) {
                 g_signal_handlers_disconnect_by_func(trashMonitor_, (gpointer)G_CALLBACK(onTrashChanged), this);
                 g_object_unref(trashMonitor_);
@@ -448,12 +462,17 @@ void PlacesModel::onVolumeChanged(GVolumeMonitor* /*monitor*/, GVolume* volume,
     PlacesModelVolumeItem* item = pThis->itemFromVolume(volume);
     if(item) {
         item->update();
+        QStandardItem* ejectBtn = item->parent()->child(item->row(), 1);
+        Q_ASSERT(ejectBtn);
         if(!item->isMounted()) { // the volume is unmounted, remove the eject button if needed
             // remove the eject button for the volume (at column 1 of the same row)
-            QStandardItem* ejectBtn = item->parent()->child(item->row(), 1);
-            Q_ASSERT(ejectBtn);
             ejectBtn->setIcon(QIcon());
         }
+        else if(ejectBtn->icon().isNull()) {
+            // this function may be called before onMountAdded(),
+            // so that the path is set but the eject icon isn't added yet
+            ejectBtn->setIcon(pThis->ejectIcon_);
+        }
     }
 }
 
