@@ -27,6 +27,7 @@ namespace Fm {
 ProxyFolderModel::ProxyFolderModel(QObject* parent):
     QSortFilterProxyModel(parent),
     showHidden_(false),
+    backupAsHidden_(true),
     folderFirst_(true),
     showThumbnails_(false),
     thumbnailSize_(0) {
@@ -38,8 +39,6 @@ ProxyFolderModel::ProxyFolderModel(QObject* parent):
 }
 
 ProxyFolderModel::~ProxyFolderModel() {
-    qDebug("delete ProxyFolderModel");
-
     if(showThumbnails_ && thumbnailSize_ != 0) {
         FolderModel* srcModel = static_cast<FolderModel*>(sourceModel());
         // tell the source model that we don't need the thumnails anymore
@@ -53,17 +52,23 @@ ProxyFolderModel::~ProxyFolderModel() {
 void ProxyFolderModel::setSourceModel(QAbstractItemModel* model) {
     if(model == sourceModel()) // avoid setting the same model twice
         return;
+    FolderModel* oldSrcModel = static_cast<FolderModel*>(sourceModel());
+#if (QT_VERSION == QT_VERSION_CHECK(5,11,0))
+    // workaround for Qt-5.11 bug https://bugreports.qt.io/browse/QTBUG-68581
+    if(oldSrcModel) {
+        disconnect(oldSrcModel, SIGNAL(destroyed()), this, SLOT(_q_sourceModelDestroyed()));
+    }
+#endif
     if(model) {
         // we only support Fm::FolderModel
         Q_ASSERT(model->inherits("Fm::FolderModel"));
 
         if(showThumbnails_ && thumbnailSize_ != 0) { // if we're showing thumbnails
-            FolderModel* oldSrcModel = static_cast<FolderModel*>(sourceModel());
-            FolderModel* newSrcModel = static_cast<FolderModel*>(model);
             if(oldSrcModel) { // we need to release cached thumbnails for the old source model
                 oldSrcModel->releaseThumbnails(thumbnailSize_);
                 disconnect(oldSrcModel, SIGNAL(thumbnailLoaded(QModelIndex, int)));
             }
+            FolderModel* newSrcModel = static_cast<FolderModel*>(model);
             if(newSrcModel) { // tell the new source model that we want thumbnails of this size
                 newSrcModel->cacheThumbnails(thumbnailSize_);
                 connect(newSrcModel, &FolderModel::thumbnailLoaded, this, &ProxyFolderModel::onThumbnailLoaded);
@@ -90,6 +95,14 @@ void ProxyFolderModel::setShowHidden(bool show) {
     }
 }
 
+void ProxyFolderModel::setBackupAsHidden(bool backupAsHidden) {
+    if(backupAsHidden != backupAsHidden_) {
+        backupAsHidden_ = backupAsHidden;
+        invalidateFilter();
+        Q_EMIT sortFilterChanged();
+    }
+}
+
 // need to call invalidateFilter() manually.
 void ProxyFolderModel::setFolderFirst(bool folderFirst) {
     if(folderFirst != folderFirst_) {
@@ -108,18 +121,20 @@ void ProxyFolderModel::setSortCaseSensitivity(Qt::CaseSensitivity cs) {
 
 bool ProxyFolderModel::filterAcceptsRow(int source_row, const QModelIndex& source_parent) const {
     if(!showHidden_) {
-        QAbstractItemModel* srcModel = sourceModel();
-        QString name = srcModel->data(srcModel->index(source_row, 0, source_parent)).toString();
-        if(name.startsWith(".") || name.endsWith("~")) {
-            return false;
+        if(FolderModel* srcModel = static_cast<FolderModel*>(sourceModel())) {
+            auto info = srcModel->fileInfoFromIndex(srcModel->index(source_row, 0, source_parent));
+            if(info && (info->isHidden() || (backupAsHidden_ && info->isBackup()))) {
+                return false;
+            }
         }
     }
     // apply additional filters if there're any
-    Q_FOREACH(ProxyFolderModelFilter* filter, filters_) {
-        FolderModel* srcModel = static_cast<FolderModel*>(sourceModel());
-        auto fileInfo = srcModel->fileInfoFromIndex(srcModel->index(source_row, 0, source_parent));
-        if(!filter->filterAcceptsRow(this, fileInfo)) {
-            return false;
+    for(ProxyFolderModelFilter* const filter : qAsConst(filters_)) {
+        if(FolderModel* srcModel = static_cast<FolderModel*>(sourceModel())){
+            auto fileInfo = srcModel->fileInfoFromIndex(srcModel->index(source_row, 0, source_parent));
+            if(!filter->filterAcceptsRow(this, fileInfo)) {
+                return false;
+            }
         }
     }
     return true;
@@ -140,17 +155,26 @@ bool ProxyFolderModel::lessThan(const QModelIndex& left, const QModelIndex& righ
             }
         }
 
+        int comp;
         switch(sortColumn()) {
         case FolderModel::ColumnFileMTime:
-            return leftInfo->mtime() < rightInfo->mtime();
+            comp = leftInfo->mtime() - rightInfo->mtime();
+            break;
         case FolderModel::ColumnFileSize:
-            return leftInfo->size() < rightInfo->size();
+            comp = leftInfo->size() - rightInfo->size();
+            break;
         default: {
             QString leftText = left.data(Qt::DisplayRole).toString();
             QString rightText = right.data(Qt::DisplayRole).toString();
-            return collator_.compare(leftText, rightText) < 0;
+            comp = collator_.compare(leftText, rightText);
+            break;
         }
         }
+        // always sort files by their display names when they have the same property
+        if(comp == 0) {
+            return collator_.compare(leftInfo->displayName(), rightInfo->displayName()) < 0;
+        }
+        return comp < 0;
     }
     return QSortFilterProxyModel::lessThan(left, right);
 }
